@@ -26,6 +26,7 @@ const Gtk = imports.gi.Gtk;
 const GtkClutter = imports.gi.GtkClutter;
 const Pango = imports.gi.Pango;
 
+const Gettext = imports.gettext;
 const _ = imports.gettext.gettext;
 
 const Lang = imports.lang;
@@ -53,6 +54,18 @@ MainToolbar.prototype = {
         this.actor = new GtkClutter.Actor({ contents: this.widget });
 
         // setup listeners to mode changes that affect the toolbar layout
+        this._searchStringId =
+            Global.searchController.connect('search-string-changed',
+                                            Lang.bind(this, this._setToolbarTitle));
+        this._searchTypeId =
+            Global.searchTypeManager.connect('active-changed',
+                                             Lang.bind(this, this._setToolbarTitle));
+        this._searchMatchId =
+            Global.searchMatchManager.connect('active-changed',
+                                              Lang.bind(this, this._setToolbarTitle));
+        this._searchSourceId =
+            Global.sourceManager.connect('active-changed',
+                                         Lang.bind(this, this._setToolbarTitle));
         this._selectionModeId =
             Global.selectionController.connect('selection-mode-changed',
                                                Lang.bind(this, this._onSelectionModeChanged));
@@ -74,6 +87,26 @@ MainToolbar.prototype = {
                     Global.selectionController.disconnect(this._selectionModeId);
                     this._selectionModeId = 0;
                 }
+
+                if (this._searchStringId != 0) {
+                    Global.searchController.disconnect(this._searchStringId);
+                    this._searchStringId = 0;
+                }
+
+                if (this._searchTypeId != 0) {
+                    Global.searchTypeManager.disconnect(this._searchTypeId);
+                    this._searchTypeId = 0;
+                }
+
+                if (this._searchMatchId != 0) {
+                    Global.searchMatchManager.disconnect(this._searchMatchId);
+                    this._searchMatchId = 0;
+                }
+
+                if (this._searchSourceId != 0) {
+                    Global.sourceManager.disconnect(this._searchSourceId);
+                    this._searchSourceId = 0;
+                }
             }));
 
         // setup listeners from toolbar actions to window mode changes
@@ -108,36 +141,14 @@ MainToolbar.prototype = {
         }
     },
 
-    _updateSelectionLabel: function() {
-        let length = Global.selectionController.getSelection().length;
-        let collection = Global.collectionManager.getActiveItem();
-        let primary = null;
-        let detail = null;
-
-        if (length == 0)
-            detail = _("Click on items to select them");
-        else
-            detail = (_("%d selected").format(length));
-
-        if (collection) {
-            primary = collection.name;
-            detail = '(' + detail + ')';
-        } else if (length != 0) {
-            primary = detail;
-            detail = null;
-        }
-
-        this.widget.set_labels(primary, detail);
-    },
-
     _populateForSelectionMode: function() {
         this.widget.set_mode(Gd.MainToolbarMode.SELECTION);
 
         // connect to selection changes while in this mode
         this._selectionChangedId =
             Global.selectionController.connect('selection-changed',
-                                               Lang.bind(this, this._updateSelectionLabel));
-        this._updateSelectionLabel();
+                                               Lang.bind(this, this._setToolbarTitle));
+        this._setToolbarTitle();
 
         this.widget.show_all();
     },
@@ -148,48 +159,95 @@ MainToolbar.prototype = {
         // connect to active collection changes while in this mode
         this._collectionId =
             Global.collectionManager.connect('active-changed',
-                                             Lang.bind(this, this._onActiveCollection));
-        this._onActiveCollection();
+                                             Lang.bind(this, this._onActiveCollectionChanged));
+        this._onActiveCollectionChanged();
 
         this.widget.show_all();
     },
 
-    _onActiveCollection: function() {
+    _onActiveCollectionChanged: function() {
         let item = Global.collectionManager.getActiveItem();
+        this.widget.set_back_visible(item != null);
 
-        if (item) {
-            this.widget.set_back_visible(true);
-            this.widget.set_labels(item.name, null);
-        } else {
-            this.widget.set_back_visible(false);
-            this.widget.set_labels(_("New and Recent"), null);
+        this._setToolbarTitle();
+    },
+
+    _setToolbarTitle: function() {
+        let mode = this.widget.get_mode();
+        let activeCollection = Global.collectionManager.getActiveItem();
+        let primary = null;
+        let detail = null;
+
+        if (mode == Gd.MainToolbarMode.OVERVIEW) {
+            if (activeCollection) {
+                primary = activeCollection.name;
+            } else {
+                let string = Global.searchController.getString();
+
+                if (string == '') {
+                    let searchType = Global.searchTypeManager.getActiveItem();
+                    let searchSource = Global.sourceManager.getActiveItem();
+
+                    if (searchType.id != 'all')
+                        primary = searchType.name;
+                    else
+                        primary = _("New and Recent");
+
+                    if (searchSource.id != 'all')
+                        detail = searchSource.name;
+                } else {
+                    let searchMatch = Global.searchMatchManager.getActiveItem();
+
+                    primary = _("Results for \"%s\"").format(string);
+                    if (searchMatch.id == 'title')
+                        detail = _("filtered by title");
+                    else if (searchMatch.id == 'author')
+                        detail = _("filtered by author");
+                }
+            }
+        } else if (mode == Gd.MainToolbarMode.PREVIEW) {
+            let doc = Global.documentManager.getActiveItem();
+            primary = doc.name;
+
+            if (this._model) {
+                let curPage, totPages;
+
+                curPage = this._model.get_page();
+                totPages = this._model.get_document().get_n_pages();
+
+                detail = _("%d of %d").format(curPage + 1, totPages);
+            }
+        } else if (mode == Gd.MainToolbarMode.SELECTION) {
+            let length = Global.selectionController.getSelection().length;
+
+            if (length == 0)
+                detail = _("Click on items to select them");
+            else
+                detail = Gettext.ngettext("%d selected",
+                                          "%d selected",
+                                          length).format(length);
+
+            if (activeCollection) {
+                primary = activeCollection.name;
+            } else if (length != 0) {
+                primary = detail;
+                detail = null;
+            }
         }
+
+        if (detail)
+            detail = '(' + detail + ')';
+
+        this.widget.set_labels(primary, detail);
     },
 
     _populateForPreview: function(model) {
         this.widget.set_mode(Gd.MainToolbarMode.PREVIEW);
-
-        this._updateModelLabels();
+        this._setToolbarTitle();
 
         this.widget.show_all();
     },
 
-    _updateModelLabels: function() {
-        let pageLabel = null;
-        let doc = Global.documentManager.getActiveItem();
-
-        if (this._model) {
-            let curPage, totPages;
-
-            curPage = this._model.get_page();
-            totPages = this._model.get_document().get_n_pages();
-
-            pageLabel = _("(%d of %d)").format(curPage + 1, totPages);
-        }
-
-        this.widget.set_labels(doc.name, pageLabel);
-    },
-
     _onWindowModeChanged: function() {
         let mode = Global.modeController.getWindowMode();
 
@@ -218,10 +276,10 @@ MainToolbar.prototype = {
         this._model = model;
         this._model.connect('page-changed', Lang.bind(this,
             function() {
-                this._updateModelLabels();
+                this._setToolbarTitle();
             }));
 
-        this._updateModelLabels();
+        this._setToolbarTitle();
     }
 };
 
